// Code based on Sivan Toledo's original published at http://www.tau.ac.il/~stoledo/lego/AVR-ir/
// Modified by Jonathan Oakley

//#include <avr/io.h>

#include <stdlib.h>
#include <stdint.h>

#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/signal.h>
#include <avr/pgmspace.h>
#include "uart.h"
#include "PFir.h"

static volatile uint32_t dummy = 0;
volatile uint32_t idleperiod = 0;


//volatile uint8_t quiet = 0;
static PFmsgS 	send1={0xA5CC,2},
				send2={0x123F,2};


void rx_msg(uint8_t ch,uint16_t cmd)
{
	if (i2c_out_msgA[ch].msg.msg==cmd && cmd!=PF_INVALID)
		i2c_out_msgA[ch].msg.cnt++; // valid message was repeated
	else
	{
		i2c_out_msgA[ch].msg.msg=cmd;
		i2c_out_msgA[ch].msg.cnt=1; // first time message seen
		if (cmd==PF_INVALID)
			pf_newrx&=~(1<<ch);		// not useful, pretend it never happened
		else
			pf_newrx|=1<<ch;		// new and valid, flag it to consumer
	}
}

void process_IR_in(void)
{
	PFcmdU pfcmd;
	uint8_t pf_chan;

	if (pf_cmd_recv_valid)
	{
		pfcmd.u16=pf_cmd_recv;
		if ((pfcmd.lrc.nibble1 ^ pfcmd.lrc.nibble2 ^ pfcmd.lrc.nibble3 ^ pfcmd.lrc.lrc) == 0xF)
		{
			// pf_cmd_recv = pf_cmd;
			pf_chan=pfcmd.direct.channel;
			if (pfcmd.direct.escape) // no mode field (Combo PWM)
			{
				rx_msg(pf_chan,pf_cmd);			// got a message for this channel
				rx_msg(pf_chan+4,PF_INVALID);	// applies to both pins of both outputs 
				rx_msg(pf_chan+8,PF_INVALID);
				rx_msg(pf_chan+12,PF_INVALID);
			}
			else
			{
				switch (pfcmd.direct.mode)
				{
					case pf_ignore:	// unused mode code, assume it's like a direct command
					case pf_direct: 	// Combo direct
						rx_msg(pf_chan,pf_cmd);			// got a message for this channel
						rx_msg(pf_chan+4,PF_INVALID);	// applies to both pins of both outputs 
						rx_msg(pf_chan+8,PF_INVALID);
						rx_msg(pf_chan+12,PF_INVALID);
						break;
					case pf_continuous:	// Single pin continuous
						pf_chan+=pfcmd.continuous.output*8+pfcmd.continuous.pin*4;
						rx_msg(pf_chan,pf_cmd);	// got a message for this channel, output and pin
						break;
					case pf_timeout:	// Single pin timeout
						pf_chan+=pfcmd.timeout.output*8+pfcmd.timeout.pin*4;
						rx_msg(pf_chan,pf_cmd);	// got a message for this channel, output and pin
						break;
					default:			// Single output
						pf_chan+=pfcmd.singlePWM.output*8;
						rx_msg(pf_chan,pf_cmd);			// got a message for this channel and output
						rx_msg(pf_chan+4,PF_INVALID);	// no such thing as the other pin... 
						break;
				}
			}

			pf_cmd_cnt++;
		}
		pf_cmd_recv_valid=0;
	}
}

void process_IR_out(void)
{
	static uint8_t nxt_tx_ch=0;

	while (q_rpt==0 && long_timer>SND_NO_BURST)	// is the queue empty, and are we allowed to transmit?	
	{
		if (nxt_tx_ch>=PF_MSG_SLOTS)	// last one in block is sent
		{
			nxt_tx_ch=0;	// rewind...
			long_timer=0;	// ...and go quiet
			break;
		}
		if (i2c_snd_msgA[nxt_tx_ch].msg.cnt) 	// do we have a message to send?
		{
			pf_send(i2c_snd_msgA[nxt_tx_ch].msg.msg,1); 	// yes, send it once
			if (i2c_snd_msgA[nxt_tx_ch].msg.cnt!=UINT8_MAX)	// unless it's continuous repeat...
				i2c_snd_msgA[nxt_tx_ch].msg.cnt--;			// that's one more done
		}
		nxt_tx_ch++;
		break;
	}
}


/*
 * Main routine.
 */

int main() 
{
/*
 *	Initialize UART library, pass baudrate and AVR cpu clock with the macro
 *		UART_BAUD_SELECT() (normal speed mode )
 *	or
 *		UART_BAUD_SELECT_DOUBLE_SPEED() ( double speed mode)
 */
	uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );

/*
 * now enable interrupt, since UART library is interrupt controlled
 */
	sei();

/*
 * Transmit single character to UART
 */
	uart_putc('\r');
	uart_putc('\n');

	pf_init();
	i2c_pullup_config();
	i2c_slave_init();

	i2c_snd_msgA[0].msg=send1;
	i2c_snd_msgA[3].msg=send2;



	while (1)
	{
		process_IR_in();
		process_IR_out();
	}
}



